import {
  select,
  csv,
  line,
  scaleLinear,
  scaleTime,
  extent,
  axisLeft,
  axisBottom,
  curveBasis,
  area,
  format
} from 'd3';

const svg = select('svg');

const width = +svg.attr('width');
const height = +svg.attr('height');

const render = data => {
  const title = 'Flows per Day';
  const xValue = d => d.timestamp;
  const xAxisLabel = 'Time';
  const yValue = d => d.count;
  const circleRadius = 6;
  const yAxisLabel = 'Count';
  const margin = { top: 60, right: 40, bottom: 88, left: 150 };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  
  // Define D3 Axis Scales
  const xScale = scaleTime()
    .domain(extent(data, xValue))
    .range([0, innerWidth]);
  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0])
    .nice();
  
  
  // Give Graph Margins
  const g = svg.append('g')
    .attr('transform', `translate(${margin.left},${margin.top})`);
  g.append('text')
      .attr('class', 'title')
      .attr('y', -10)
      .attr('x', 160)
      .text(title);
  
  
  // Adjust Axis Ticks
  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight)
    .tickPadding(15)
    .ticks(6);
  
  const yAxisTickFormat = number =>
    format('.2s')(number);//.replace('0.0', '0');
  const yAxis = axisLeft(yScale)
    .tickFormat(yAxisTickFormat)
    .tickSize(-innerWidth)
    .tickPadding(10);
  
  
  // Axis Groups for Text and Customizations
  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove();
  yAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', -93)
      .attr('x', -innerHeight / 2)
      .attr('fill', 'black')
      .attr('transform', `rotate(-90)`)
      .attr('text-anchor', 'middle')
      .text(yAxisLabel);
  
  const xAxisG = g.append('g').call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);
  xAxisG.select('.domain').remove();
  xAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', 75)
      .attr('x', innerWidth / 2)
      .attr('fill', 'black')
      .text(xAxisLabel);
  
  
  // Plot Data
  // Time Series Scatter Plot
  //g.selectAll('circle').data(data)
  //  .enter().append('circle')
  //    .attr('cy', d => yScale(yValue(d)))
  //    .attr('cx', d => xScale(xValue(d)))
  //    .attr('r', circleRadius);
  
  // Time Series Line Chart
  const lineGenerator = line()
    .x(d => xScale(xValue(d)))
    .y(d => yScale(yValue(d)))
    .curve(curveBasis);
  g.append('path')
    .attr('class', 'line-path')
    .attr('d', lineGenerator(data));
  
  // Time Series Area Chart
  //const areaGenerator = area()
  //  .x(d => xScale(xValue(d)))
  //  .y1(d => yScale(yValue(d)))
  //  .y0(innerHeight)
  //  .curve(curveBasis);
  //g.append('path')
  //  .attr('class', 'area-path')
  //  .attr('d', areaGenerator(data));
  
};

csv('data.csv')
  .then(data => {
    data.forEach(d => {
      d.count = +d.count;
      d.timestamp = new Date(d.timestamp);
    });
    render(data);
  });